local folder = "GemCensus_Display"
local core = LibStub("AceAddon-3.0"):GetAddon("GemCensus")
local display = core.display

local _G = _G
local GetMarketValue = false
local GetBestMatch = false
local GetSellValue = GetSellValue
local AucAdvanced = false
local tonumber = tonumber
local iLink = core.iLink
local getmetatable = getmetatable
local setmetatable = setmetatable
local Debug = core.Debug

display.auctionPrice = {
	[true] = setmetatable({}, {
		__index = function(t,itemID)
			t[itemID] = display:GetAuctionValue(itemID, true)
			return t[itemID]
		end
	}),
	[false] = setmetatable({}, {
		__index = function(t,itemID)
			t[itemID] = display:GetAuctionValue(itemID)
			return t[itemID]
		end
	})
}


local prev_OnInitialize = display.OnInitialize
function display:OnInitialize()
	if prev_OnInitialize then
		prev_OnInitialize(self)
	end
	
	AucAdvanced = AucAdvanced or _G.AucAdvanced
	GetMarketValue = GetMarketValue or AucAdvanced and AucAdvanced.API.GetMarketValue
	GetBestMatch = GetBestMatch or AucAdvanced and AucAdvanced.API.GetBestMatch
end

local prev_OnEnable = display.OnEnable
function display:OnEnable()
	if prev_OnEnable then
		prev_OnEnable(self)
	end

	self.auctionPrice[true] 			= setmetatable({}, getmetatable(self.auctionPrice[true]))
	self.auctionPrice[false] 			= setmetatable({}, getmetatable(self.auctionPrice[false]))
end

function display:CanGetPrices()
	return _G.AucAdvanced and true or false
end

function display:GetAuctionValue(itemID, priceMatch)
	local itemLink = iLink[itemID]
	
--~ 	Debug("GetAuctionValue A", itemID, itemLink)
	if not itemLink then return 0; end
	local itemValue;
	local itemValueAuctioneer, itemValueAuctioneer2 = 0, 0;


	if GetMarketValue then
		itemValueAuctioneer = GetMarketValue(itemLink);
	end
--~ 	Debug("GetAuctionValue B", itemID, itemLink, itemValueAuctioneer)
	
	if priceMatch == true  then
		itemValueAuctioneer2 = GetBestMatch and GetBestMatch(itemLink, itemValueAuctioneer) or 0
		if itemValueAuctioneer2 and itemValueAuctioneer2 > 0 then
			itemValueAuctioneer = itemValueAuctioneer2
		end
	end
--~ 	Debug("GetAuctionValue B", itemID, itemLink, itemValueAuctioneer2)
	

	return tonumber(itemValueAuctioneer or 0);
end

function display:GetCheapestAucValue(itemID)
	if not AucAdvanced then
		return 0
	end
	local results = AucAdvanced.API.QueryImage({
		itemId = itemID,
	})
	local Const = AucAdvanced.Const
	local count = #results
	
--~ 	Debug("GetCheapestAucValue", iLink[itemID], count) --, i, buyOut
	
	local lowestValue = 0
	local result
	local buyOut
	for i=1, count do 
		result = results[i]
		buyOut = result[Const.BUYOUT]

		if lowestValue == 0 or buyOut < lowestValue then
			lowestValue = buyOut
		end
	end
	return lowestValue
end


function display:CanUndercutEveryone(itemID, ourValue)
--~ 	local ourValue = ourValue or self.meta_AuctionPriceMatching[itemID]
	local ourValue = ourValue or self.auctionPrice[true][itemID]


	local lowestValue = self:GetCheapestAucValue(itemID)
	
--~ 	Debug("CanUndercutEveryone", iLink[itemID], ourValue, lowestValue)
	
	if lowestValue == 0 or ourValue < lowestValue then
		return true
	end
	return false
end

